@@ -1,6 +1,6 @@
 from . import utils
 from .embed import Embed
 from .identifiers import SubOption
-from .models import Message
+from .models import Message, Attachment
 from .package import Package
 from .server import DisunityServer
