@@ -1,5 +1,7 @@
 import asyncio
 import importlib
+import json
+import mimetypes
 import time
 
 import aiohttp
@@ -46,6 +48,7 @@ class DisunityServer(quart.Quart):
         self.config["BOT_TOKEN"] = bot_token
         self.config["CREDENTIALS_EXPIRE_IN"] = 0  # set on start up
         self.__cache: cache.ApplicationCache = cache.ApplicationCache()
+        self.__packages = dict()
         self.__key = VerifyKey(bytes.fromhex(self.config["CLIENT_PUBLIC_KEY"]))
         self.add_url_rule(
             "/interactions", "interactions", self.interactions, methods=["POST"]
@@ -54,6 +57,12 @@ class DisunityServer(quart.Quart):
     @property
     def cache(self) -> cache.ApplicationCache:
         return self.__cache
+    
+    def get_package(self, package_name):
+        return self.__packages.get(package_name, None)
+    
+    def get_package_names(self) -> list[str]:
+        return list(self.__packages.keys())
 
     def error_handler(self, exc: Exception):
         raise exc
@@ -97,6 +106,7 @@ class DisunityServer(quart.Quart):
         url: str,
         headers: None | dict = None,
         payload: None | dict = None,
+        files: None | dict = None,
         override_checks: bool = False,
     ):
         """
@@ -105,7 +115,6 @@ class DisunityServer(quart.Quart):
         ----------
         method: str
             The method to use. Can be either: GET, PUT, PATCH, DELETE, POST
-
         url: str
             The url to make the request to. If performing a request to discord, do not include
             'https://discord.com/api/v10', it will be automatically added.
@@ -114,21 +123,42 @@ class DisunityServer(quart.Quart):
             client credentials will be used.
         payload: Optional[dict]
             The payload to send to the url. Only applicable for POST, PUT, and PATCH requests.
+        files: Optional[list]
+            A list of dictionaries with 'id', 'filename' and 'content'.
         """
-
         if not url.startswith("https://"):
             url = "https://discord.com/api/v10/" + url
 
-        if headers is not None:
-            _headers = headers
-        elif headers is None and self.config["BOT_TOKEN"] is not None:
-            _headers = {"Authorization": "Bot " + self.config["BOT_TOKEN"]}
+        if headers is None and self.config["BOT_TOKEN"] is not None:
+            headers = {"Authorization": "Bot " + self.config["BOT_TOKEN"]}
+        elif headers is None:
+            headers = self.auth()
+
+        if files:
+            data = aiohttp.FormData()
+            if payload:
+                data.add_field(
+                    "payload_json",
+                    json.dumps(payload),
+                    content_type="application/json",
+                )
+            for file in files:
+                content_type, _ = mimetypes.guess_type(file["filename"])
+                if content_type is None:
+                    content_type = "application/octet-stream"
+                data.add_field(
+                    f"files[{file['id']}]",
+                    file["content"],
+                    filename=file["filename"],
+                    content_type=content_type,
+                )
+            request_kwargs = {"data": data}
         else:
-            _headers = self.auth()
+            request_kwargs = {"json": payload} if payload else {}
 
         async with aiohttp.ClientSession() as session:
             async with session.request(
-                method, url, headers=_headers, json=payload
+                method, url, headers=headers, **request_kwargs
             ) as maybe_response:
                 if override_checks:
                     return maybe_response
@@ -151,10 +181,41 @@ class DisunityServer(quart.Quart):
         setup(self)
 
     def register_package(self, package_class):
+        self.__packages[package_class.__class__.__name__] = package_class
         contents = package_class.unpack()
         for item in contents:
             self.__cache.add_item(item)
 
+    async def global_check(self, context: Context) -> bool:
+        """Global check for all application commands, message components and modal submits.
+
+        Args:
+            context (Context): Application command context.
+
+        Returns:
+            bool: Command is executed only if `True` is returned.
+                    `True` is returned by default.
+        """
+        return True
+    
+    async def global_before_interaction(self, context: Context):
+        """Called before the execution of any application command, message components and modal submits.
+        This is called after global check is handled.
+
+        Args:
+            context (Context): Application command or message component context.
+        """
+        pass
+
+    async def global_after_interaction(self, context: Context):
+        """Called after the execution of any application command, message components, and modal submits.
+
+        Args:
+            context (Context): Application command or message component context.
+        """
+        pass
+
+
     async def interactions(self):
         self.verify(request)
         received = request.json
@@ -204,6 +265,15 @@ class DisunityServer(quart.Quart):
             ctx: Context = Context(self, received)
             if coroutine is None:
                 raise errors.CommandNotFound(ctx.command_name)
+            
+            check = await self.global_check(ctx)
+            if check != True:
+                if isinstance(check, dict) and "type" in check:
+                    return jsonify(check)
+                else:
+                    return jsonify({"type": utils.InteractionCallbackTypes.PONG})
+            
+            await self.global_before_interaction(ctx)
 
             if coroutine.ack:
                 response = {
@@ -216,10 +286,15 @@ class DisunityServer(quart.Quart):
                     }
 
                 ctx.acked = True
-                asyncio.create_task(coroutine(ctx))
+                async def combined_task(ctx):
+                    await coroutine(ctx)
+                    await self.global_after_interaction(ctx)
+
+                asyncio.create_task(combined_task(ctx))
                 return jsonify(response)
 
             response = await coroutine(ctx)
+            asyncio.create_task(self.global_after_interaction(ctx))
             return jsonify(response)
 
         elif received["type"] == utils.InteractionTypes.MESSAGE_COMPONENT:
@@ -231,6 +306,15 @@ class DisunityServer(quart.Quart):
             if component is None:
                 raise errors.ComponentNotFound(context.custom_id.split("-")[0])
 
+            check = await self.global_check(context)
+            if check != True:
+                if isinstance(check, dict) and "type" in check:
+                    return jsonify(check)
+                else:
+                    return jsonify({"type": utils.InteractionCallbackTypes.PONG})
+            
+            await self.global_before_interaction(context)
+
             if component.ack:
                 response = {
                     "type": utils.InteractionCallbackTypes.DEFERRED_UPDATE_MESSAGE
@@ -242,10 +326,15 @@ class DisunityServer(quart.Quart):
                     }
 
                 context.acked = True
-                asyncio.create_task(component(context))
+                async def combined_task(context):
+                    await component(context)
+                    await self.global_after_interaction(context)
+
+                asyncio.create_task(combined_task(context))
                 return jsonify(response)
 
             maybe_response = await component(context)
+            asyncio.create_task(self.global_after_interaction(context))
             return jsonify(maybe_response)
 
         elif (
@@ -276,5 +365,15 @@ class DisunityServer(quart.Quart):
             if component is None:
                 raise errors.ComponentNotFound(context.custom_id.split("-")[0])
 
+            check = await self.global_check(context)
+            if check != True:
+                if isinstance(check, dict) and "type" in check:
+                    return jsonify(check)
+                else:
+                    return jsonify({"type": utils.InteractionCallbackTypes.PONG})
+            
+            await self.global_before_interaction(context)
+
             maybe_response = await component(context)
+            asyncio.create_task(self.global_after_interaction(context))
             return jsonify(maybe_response)
